@@ -102,23 +102,20 @@
 #include <utmp.h>
 #include <setjmp.h>
 #include <stdlib.h>
-#include <string.h>
 #include <sys/syslog.h>
 #include <sys/sysmacros.h>
+#include <linux/major.h>
 #include <netdb.h>
 #ifdef HAVE_LIBAUDIT
 # include <libaudit.h>
 #endif
+
 #include "pathnames.h"
 #include "my_crypt.h"
 #include "login.h"
 #include "xstrncpy.h"
 #include "nls.h"
 
-#include <sys/sysmacros.h>
-#include <linux/major.h>
-
-#include <utmp.h>
 
 #ifdef HAVE_SECURITY_PAM_MISC_H
 #  include <security/pam_appl.h>
@@ -342,6 +339,34 @@ logaudit(const char *tty, const char *username, const char *hostname,
 # define logaudit(tty, username, hostname, pwd, status)
 #endif /* HAVE_LIBAUDIT */
 
+#ifdef HAVE_SECURITY_PAM_MISC_H
+/* encapsulate stupid "void **" pam_get_item() API */
+int
+get_pam_username(pam_handle_t *pamh, char **name)
+{
+	const void *item = (void *) *name;
+	int rc;
+	rc = pam_get_item(pamh, PAM_USER, &item);
+	*name = (char *) item;
+	return rc;
+}
+#endif
+
+/*
+ * We need to check effective UID/GID. For example $HOME could be on root
+ * squashed NFS or on NFS with UID mapping and access(2) uses real UID/GID.
+ * The open(2) seems as the surest solution.
+ * -- kzak@redhat.com (10-Apr-2009)
+ */
+int
+effective_access(const char *path, int mode)
+{
+	int fd = open(path, mode);
+	if (fd != -1)
+		close(fd);
+	return fd == -1 ? -1 : 0;
+}
+
 int
 main(int argc, char **argv)
 {
@@ -353,7 +378,7 @@ main(int argc, char **argv)
     int ask, fflag, hflag, pflag, cnt, errsv;
     int quietlog, passwd_req;
     char *domain, *ttyn;
-    char tbuf[MAXPATHLEN + 2], tname[sizeof(_PATH_TTY) + 10];
+    char tbuf[MAXPATHLEN + 2], tname[sizeof(_PATH_DEV_TTY) + 10];
     char *termenv;
     char *childArgv[10];
     char *buff;
@@ -393,7 +418,7 @@ main(int argc, char **argv)
      */
     gethostname(tbuf, sizeof(tbuf));
     xstrncpy(thishost, tbuf, sizeof(thishost));
-    domain = index(tbuf, '.');
+    domain = strchr(tbuf, '.');
 
     username = tty_name = hostname = NULL;
     fflag = hflag = pflag = 0;
@@ -412,7 +437,7 @@ main(int argc, char **argv)
 	      exit(1);
 	  }
 	  hflag = 1;
-	  if (domain && (p = index(optarg, '.')) &&
+	  if (domain && (p = strchr(optarg, '.')) &&
 	      strcasecmp(p, domain) == 0)
 	    *p = 0;
 
@@ -474,7 +499,7 @@ main(int argc, char **argv)
 
     if (ttyn == NULL || *ttyn == '\0') {
 	/* no snprintf required - see definition of tname */
-	sprintf(tname, "%s??", _PATH_TTY);
+	snprintf(tname, sizeof(tname), "%s??", _PATH_DEV_TTY);
 	ttyn = tname;
     }
 
@@ -542,7 +567,7 @@ main(int argc, char **argv)
      * Therefore, we are safe not setting it to anything
      */
 
-    retcode = pam_start("login",username, &conv, &pamh);
+    retcode = pam_start(hflag?"remote":"login",username, &conv, &pamh);
     if(retcode != PAM_SUCCESS) {
 	fprintf(stderr, _("%s: PAM failure, aborting: %s\n"),
 		"login", pam_strerror(pamh, retcode));
@@ -591,7 +616,7 @@ main(int argc, char **argv)
 	int failcount=0;
 
 	/* if we didn't get a user on the command line, set it to NULL */
-	pam_get_item(pamh,  PAM_USER, (const void **) &username);
+	get_pam_username(pamh, &username);
 
 	/* there may be better ways to deal with some of these
 	   conditions, but at least this way I don't think we'll
@@ -605,7 +630,7 @@ main(int argc, char **argv)
 	       (retcode == PAM_USER_UNKNOWN) ||
 	       (retcode == PAM_CRED_INSUFFICIENT) ||
 	       (retcode == PAM_AUTHINFO_UNAVAIL))) {
-	    pam_get_item(pamh, PAM_USER, (const void **) &username);
+	    get_pam_username(pamh, &username);
 
 	    syslog(LOG_NOTICE,_("FAILED LOGIN %d FROM %s FOR %s, %s"),
 		   failcount, hostname, username, pam_strerror(pamh, retcode));
@@ -618,7 +643,7 @@ main(int argc, char **argv)
 	}
 
 	if (retcode != PAM_SUCCESS) {
-	    pam_get_item(pamh, PAM_USER, (const void **) &username);
+	    get_pam_username(pamh, &username);
 
 	    if (retcode == PAM_MAXTRIES)
 		syslog(LOG_NOTICE,_("TOO MANY LOGIN TRIES (%d) FROM %s FOR "
@@ -654,7 +679,7 @@ main(int argc, char **argv)
      * Grab the user information out of the password file for future usage
      * First get the username that we are actually using, though.
      */
-    retcode = pam_get_item(pamh, PAM_USER, (const void **) &username);
+    retcode = get_pam_username(pamh, &username);
     PAM_FAIL_CHECK;
 
     if (!username || !*username) {
@@ -875,7 +900,7 @@ main(int argc, char **argv)
 		sprintf(tmpstr, "%s/%s", pwd->pw_dir, _PATH_HUSHLOGIN);
 		setregid(-1, pwd->pw_gid);
 		setreuid(0, pwd->pw_uid);
-		quietlog = (access(tmpstr, R_OK) == 0);
+		quietlog = (effective_access(tmpstr, O_RDONLY) == 0);
 		setuid(0); /* setreuid doesn't do it alone! */
 		setreuid(ruid, 0);
 		setregid(-1, egid);
@@ -1226,7 +1251,7 @@ Michael Riepe <michael@stud.uni-hannover.de>
 	childArgv[childArgc++] = buff;
     } else {
 	tbuf[0] = '-';
-	xstrncpy(tbuf + 1, ((p = rindex(pwd->pw_shell, '/')) ?
+	xstrncpy(tbuf + 1, ((p = strrchr(pwd->pw_shell, '/')) ?
 			   p + 1 : pwd->pw_shell),
 		sizeof(tbuf)-1);
 
@@ -1422,7 +1447,13 @@ dolastlog(int quiet) {
 	    lseek(fd, (off_t)pwd->pw_uid * sizeof(ll), SEEK_SET);
 	}
 	memset((char *)&ll, 0, sizeof(ll));
-	time(&ll.ll_time);
+
+	{
+		time_t t;
+		time(&t);
+		ll.ll_time = t; /* ll_time is always 32bit */
+	}
+
 	xstrncpy(ll.ll_line, tty_name, sizeof(ll.ll_line));
 	if (hostname)
 	    xstrncpy(ll.ll_host, hostname, sizeof(ll.ll_host));
